@@ -20,7 +20,7 @@
 #include "screen_queue.h"
 #include "screen_interface.h"
 #include "screen_file.h"
-#include "screen_message.h"
+#include "screen_status.h"
 #include "screen_find.h"
 #include "config.h"
 #include "i18n.h"
@@ -268,7 +268,7 @@ playlist_save(struct mpdclient *c, char *name, char *defaultname)
 			filename=g_strstrip(filename);
 	} else
 #endif
-			filename=g_strdup(name);
+		filename=g_strdup(name);
 
 	if (filename == NULL)
 		return -1;
@@ -287,14 +287,14 @@ playlist_save(struct mpdclient *c, char *name, char *defaultname)
 		    mpd_connection_get_server_error(connection) == MPD_SERVER_ERROR_EXIST &&
 		    mpd_connection_clear_error(connection)) {
 			char *buf;
-			int key;
+			bool replace;
 
 			buf = g_strdup_printf(_("Replace %s [%s/%s] ? "),
 					      filename, YES, NO);
-			key = tolower(screen_getch(buf));
+			replace = screen_get_yesno(buf, false);
 			g_free(buf);
 
-			if (key != YES[0]) {
+			if (!replace) {
 				g_free(filename_utf8);
 				g_free(filename);
 				screen_status_printf(_("Aborted"));
@@ -554,7 +554,7 @@ screen_queue_update(struct mpdclient *c)
 		screen_queue_restore_selection();
 	else
 		/* the queue size may have changed, even if we havn't
-		   revceived the QUEUE idle event yet */
+		   received the QUEUE idle event yet */
 		list_window_set_length(lw, playlist_length(playlist));
 
 	if (((c->events & MPD_IDLE_PLAYER) != 0 &&
@@ -700,7 +700,10 @@ screen_queue_cmd(struct mpdclient *c, command_t cmd)
 		break;
 #endif
 	case CMD_SCREEN_SWAP:
-		screen_swap(c, playlist_get(&c->playlist, lw->selected));
+		if (playlist_length(&c->playlist) > 0)
+			screen_swap(c, playlist_get(&c->playlist, lw->selected));
+		else
+			screen_swap(c, NULL);
 		return true;
 
 	default:
@@ -765,6 +768,10 @@ screen_queue_cmd(struct mpdclient *c, command_t cmd)
 		lw->selected--;
 		lw->range_base--;
 
+		if (lw->range_selection)
+			list_window_scroll_to(lw, lw->range_base);
+		list_window_scroll_to(lw, lw->selected);
+
 		screen_queue_save_selection();
 		return true;
 
@@ -779,6 +786,10 @@ screen_queue_cmd(struct mpdclient *c, command_t cmd)
 		lw->selected++;
 		lw->range_base++;
 
+		if (lw->range_selection)
+			list_window_scroll_to(lw, lw->range_base);
+		list_window_scroll_to(lw, lw->selected);
+
 		screen_queue_save_selection();
 		return true;
 
